@@ -2,8 +2,8 @@
 __title__ = "sqlite3-to-mysql"
 __description__ = "A simple Python tool to transfer data from SQLite 3 to MySQL"
 __url__ = "https://github.com/techouse/sqlite3-to-mysql"
-__version__ = "1.3.4"
-__author__ = "Klemen Tušar"
+__version__ = "1.3.6"
+__author__ = "Klemen Tusar"
 __author_email__ = "techouse@gmail.com"
 __license__ = "MIT"
-__copyright__ = "Copyright 2020 Klemen Tušar"
+__copyright__ = "Copyright 2020 Klemen Tusar"
